@@ -4,6 +4,46 @@
 #include "util/mallocHelper.h"
 #include "util/smbiosHelper.h"
 
+#include <pdh.h>
+
+static void ffPdhOpenCloseQuery(HQUERY* query)
+{
+    assert(query);
+    if (*query)
+    {
+        PdhCloseQuery(*query);
+        *query = NULL;
+    }
+}
+
+static const char* detectThermalTemp(double* result)
+{
+    // typeperf.exe -sc 1 "\Thermal Zone Information(*)\Temperature"
+
+    __attribute__((__cleanup__(ffPdhOpenCloseQuery))) HQUERY query = NULL;
+
+    if (PdhOpenQuery(NULL, 0, &query) != ERROR_SUCCESS)
+        return "Failed to open PDH query";
+
+    HCOUNTER counter = NULL;
+    if (PdhAddEnglishCounter(query,
+        L"\\Thermal Zone Information(*)\\Temperature",
+        0,
+        &counter) != ERROR_SUCCESS)
+        return "Failed to add TZI temperature counter";
+
+    if (PdhCollectQueryData(query) != ERROR_SUCCESS)
+        return "Failed to collect query data";
+
+    PDH_FMT_COUNTERVALUE value;
+    if (PdhGetFormattedCounterValue(counter, PDH_FMT_DOUBLE, NULL, &value) != ERROR_SUCCESS)
+        return "Failed to format counter value";
+
+    *result = value.doubleValue - 273;
+
+    return NULL;
+}
+
 // 7.5
 typedef struct FFSmbiosProcessorInfo
 {
@@ -100,9 +140,7 @@ static const char* detectNCores(FFCPUResult* cpu)
         ptr = (SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX*)(((uint8_t*)ptr) + ptr->Size)
     )
     {
-        if (ptr->Relationship == RelationProcessorCore)
-            ++cpu->coresPhysical;
-        else if (ptr->Relationship == RelationGroup)
+        if (ptr->Relationship == RelationGroup)
         {
             for (uint32_t index = 0; index < ptr->Group.ActiveGroupCount; ++index)
             {
@@ -110,6 +148,10 @@ static const char* detectNCores(FFCPUResult* cpu)
                 cpu->coresLogical += ptr->Group.GroupInfo[index].MaximumProcessorCount;
             }
         }
+        else if (ptr->Relationship == RelationProcessorCore)
+            ++cpu->coresPhysical;
+        else if (ptr->Relationship == RelationProcessorPackage)
+            cpu->packages++;
     }
 
     return NULL;
@@ -126,9 +168,13 @@ static const char* detectByRegistry(FFCPUResult* cpu)
 
     if (cpu->coresLogical == 0)
     {
-        DWORD cores;
-        if (RegQueryInfoKeyW(HKEY_LOCAL_MACHINE, L"HARDWARE\\DESCRIPTION\\System\\CentralProcessor", NULL, NULL, &cores, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
-            cpu->coresOnline = cpu->coresPhysical = cpu->coresLogical = (uint16_t) cores;
+        FF_HKEY_AUTO_DESTROY hProcsKey = NULL;
+        if (ffRegOpenKeyForRead(HKEY_LOCAL_MACHINE, L"HARDWARE\\DESCRIPTION\\System\\CentralProcessor", &hProcsKey, NULL))
+        {
+            uint32_t cores;
+            if (ffRegGetNSubKeys(hProcsKey, &cores, NULL))
+                cpu->coresOnline = cpu->coresPhysical = cpu->coresLogical = (uint16_t) cores;
+        }
     }
 
     uint32_t mhz;
@@ -159,8 +205,6 @@ static const char* detectCoreTypes(FFCPUResult* cpu)
     return NULL;
 }
 
-const char* detectThermalTemp(double* current, double* critical);
-
 const char* ffDetectCPUImpl(const FFCPUOptions* options, FFCPUResult* cpu)
 {
     detectNCores(cpu);
@@ -176,7 +220,7 @@ const char* ffDetectCPUImpl(const FFCPUOptions* options, FFCPUResult* cpu)
         detectMaxSpeedBySmbios(cpu);
 
     if(options->temp)
-        detectThermalTemp(&cpu->temperature, NULL);
+        detectThermalTemp(&cpu->temperature);
 
     return NULL;
 }
